(** Pretty printing of specifications in abstract syntax (multi-process) *)
  
(**  prefixes of identifiers:
  - [c_]  constant symbol
  - [r_]  reserved function symbol (including type coercion)
  - [t_]  type (predicate)
  - [v_]  value
  - [P_]  parameter
  - [V_]  private variable
  - [Q_]  quantified variable
*)


open Abstract;;
open Symbols;;

open Clause;;


(** Separators for [pp_list]. *)

(** standard comma separation to use for [pp_list] *)
val comma_sep : unit -> unit

(** standard semicolumn separation to use for [pp_list] *)
val semicol_sep : unit -> unit

(** standard null separation to use for [pp_list] *)
val null_sep : unit -> unit

(** standard new line separation to use for [pp_list] *)
val newline_sep : unit -> unit

(** double new line separation to use for pp_list *)
val newline2_sep : unit -> unit


(** Pretty printing functions for clauses aand clauses components. *)

(** [hornpp_id sl i]
  prints the given identifier [i]
  using the given symbol list [sl].
  see above for the prefixes of identifiers. *)
val hornpp_id : eva_symbol_list -> eva_id -> unit

(** [hornpp_lid sl i]
  prints the given located identifier [i]
  using the given symbol list [sl].
  see above for the prefixes of identifiers. *)
val hornpp_lid : eva_symbol_list -> eva_lid -> unit

(** [hornpp_label l]
  prints the given label [l]. *)
val hornpp_label : eva_label -> unit

(** [hornpp_label l]
  prints the given located label [l]. *)
val hornpp_llabel : eva_llabel -> unit


(** [hornpp_type t]
  prints the given type [t]. *)
val hornpp_type : eva_type -> unit

(** [hornpp_ltype t]
  prints the given located type [t]. *)
val hornpp_ltype : eva_ltype -> unit  

(** [hornpp_scope s]
  prints the given scope [s]. *)  
val hornpp_scope : eva_scope -> unit

(** [hornpp_lscope s]
  prints the given located scope [s]. *)  
val hornpp_lscope : eva_lscope -> unit

(** [hornpp_value v]
  prints the given value [v]. *)  
val hornpp_value : eva_value -> unit

(** [hornpp_lvalue v]
  prints the given located value [v]. *)  
val hornpp_lvalue : eva_lvalue -> unit

(** [hornpp_term sl t]
  prints the given term [t]
  using the given symbol list [sl].*)
val hornpp_term : eva_symbol_list -> eva_term -> unit

(** [hornpp_lterm sl t]
  prints the given located term [t]
  using the given symbol list [sl].*)
val hornpp_lterm : eva_symbol_list -> eva_lterm -> unit

(** [hornpp_literal sign sl l]
  prints the given literal l
  preceeded by the given string [sign]
  and using the given symbol list [sl]. *) 
val hornpp_literal : string -> eva_symbol_list -> literal -> unit

(** [hornpp_tclause sl c]
  print the given tclause [c] 
  using the given symbol list [sl]. *)
val hornpp_tclause : eva_symbol_list -> tclause -> unit

(** [hornpp_clause sl c]
  print the given clause [c] 
  using the given symbol list [sl]. *)
val hornpp_clause : eva_symbol_list -> clause -> unit


(** pretty print the given specification 
  as a set of Horn clauses. *)
(* val hornpp_spec : eva_spec -> unit *)



